﻿--[[--------------------------------------------------------------------------------------------------------------------   
  BGL-TTM - Travelers Tundra Mammoth
  Copyright © 2016-2020 Campaign Builders Guild, Inc.
  <www.cbguild.com>

  License: GNU General Public License version 3 (or higher) 

  This file is part of BGL-TTM, a World of Warcraft Addon.

    BGL-TTM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BGL-TTM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BGL-TTM.  If not, see <http://www.gnu.org/licenses/>.

  Future Plans:

    - Translate phrases for all Curseforge-supported languages
    - ?? Add whitelist of other eject-enabled vehicles
    - ?? Add whitelist/blacklist of player passengers by name

--]]--------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------
-- constants
------------------------------------------------------------------------------------------------------------------------

local BGL_TTM_INTERNAL_VERSION = "8.3"
BGL_TTM_EXTERNAL_VERSION = "Version " .. BGL_TTM_INTERNAL_VERSION
BGL_TTM_ADDON_NAME = "BGL-TTM"

local FOLDER_NAME, private = ...
local L = private.L

-- local bglDEBUG = true

BGL_TTM_ADDON_DESCRIPTION = L['ADDON_INTRODUCTION'].."\n\n"
BGL_TTM_ADDON_DESCRIPTION1 = L['ADDON_DESCRIPTION_LONG']
BGL_TTM_AUTOEJECT_SHORT_DESC = L['AUTO_EJECT_ON_SUMMON']
BGL_TTM_AUTOEJECT_LONG_DESC = L['AUTO_EJECT_ON_SUMMON_LONG']
BGL_TTM_CHOOSE_EJECTION_SEATS = L['CHOOSE_EJECTION_SEATS']
BGL_TTM_SHOW_VEHICLE_SHORT_DESC = L['SHOW_VEHICLE_EJECT']
BGL_TTM_LIMIT_SHOW_VEHICLE_SHORT_DESC = L['LIMIT_SHOW_VEHICLE_EJECT']
BGL_TTM_LEFT_SEAT = L['LEFT_SEAT']
BGL_TTM_RIGHT_SEAT = L['RIGHT_SEAT']
BGL_TTM_VERSION = L['VERSION'].." "
BGL_TTM_ADDON_ACKNOWLEDGEMENT = "Supporters of this Addon:\n  Campaign Builders Guild, Inc. <www.cbguild.com>\n  Curse Inc. <www.curse.com>"

local BGL_TTM_COLOR_CODE = {}
BGL_TTM_COLOR_CODE.COLOR = {
  ["Red"] = "|cFFFF0000",
  ["Orange"] = "|cFFFF8800",
  ["Yellow"] = "|cFFFFFF00",
  ["Green"] = "|cFF00FF00",
  ["Blue"] = "|cFF0000FF",
  ["Violet"] = "|cFF8800FF",
  ["White"] = "|cFFFFFFFF",
  ["Gray"] = "|cFFAAAAAA",
  ["Dark Gray"] = "|cFF888888",
  ["Black"] = "|cFF000000"
}

local BGL_TTM_SPELL_ID1 =  61447  -- Traveler's Tundra Mammoth
local BGL_TTM_SPELL_ID2 =  62809  -- Reins of the Grand Expedition Yak
local BGL_TTM_SPELL_ID3 =  61425  -- Traveler's Tundra Mammoth
local BGL_TTM_SPELL_ID4 = 122708  -- Reins of the Grand Expedition Yak

local BGL_TTM_VENDOR_NAMES = {
   "Hakmud of Argus", 
   "Drix Blackwrench",
   "Gnimo", 
   "Mojodishu",
   "Cousin Slowhands",
   "Mystic Birdhat"
}

------------------------------------------------------------------------------------------------------------------------
-- Variables
------------------------------------------------------------------------------------------------------------------------

bglTTMData = {
  default = {
    autoEject = false,
    showVehicleEject = true,
    showVehicleEjectOnlyForTTMAndGEY = false,
    autoEjectAffectedSeats = {true, true},
  },
  vendorsOnBoard = 2,
  mountIsTTMOrGEY = false,
}

bglTTMSaveData = {
  autoEject = false,
  showVehicleEject = true,
  showVehicleEjectOnlyForTTMAndGEY = false,
  autoEjectAffectedSeats = {true, true},
}

------------------------------------------------------------------------------------------------------------------------
-- Vehicle Indicator Frame Eject Button
------------------------------------------------------------------------------------------------------------------------

function bglTTMCreateVehicleEjectButton(parentFrame)
  if 
    (not bglTTMVehicleEjectButton)
    and (not (parentFrame == nil))
  then
    CreateFrame("Button", "bglTTMVehicleEjectButton", parentFrame, "OptionsButtonTemplate")
    bglTTMVehicleEjectButton:SetMovable(false)
    bglTTMVehicleEjectButton:SetPoint("CENTER",parentFrame,"CENTER",0,0)
    bglTTMVehicleEjectButton:SetText(L['EJECT'])

    bglTTMVehicleEjectButton:SetScript("OnClick", function(self,motion)
      bglTTMEjectPassengersNow()
    end)

    parentFrame:HookScript("OnUpdate",function(self,elapsed) 
      if 
        bglTTMSaveData.showVehicleEject == true 
        and 
          (
            bglTTMData.mountIsTTMOrGEY
            or not bglTTMSaveData.showVehicleEjectOnlyForTTMAndGEY
          )
      then
        bglTTMVehicleEjectButton:Show()
      else
        bglTTMVehicleEjectButton:Hide()
      end
    end)

  end
end

------------------------------------------------------------------------------------------------------------------------
-- Utility Functions
------------------------------------------------------------------------------------------------------------------------

function bglTTMInitialize()
  bglTTMInstallVehicleEjectButton()
end

------------------------------------------------------------------------------------------------------------------------

function bglTTMIsSeatAffected(seatName)
  if (bglTTMSaveData.autoEjectAffectedSeats == nil) then
    bglTTMSaveData.autoEjectAffectedSeats = bglTTMData.default.autoEjectAffectedSeats
  end

  return (
    ( (seatName == "Left") and (bglTTMSaveData.autoEjectAffectedSeats[1] == true) )
    or ( (seatName == "Right") and (bglTTMSaveData.autoEjectAffectedSeats[2] == true) )
  )
end

------------------------------------------------------------------------------------------------------------------------

function bglTTMIsVendorPassenger()
  local i = 0
  local targetName, _ = UnitName("target")

  if UnitInVehicle("target") then
    for _, name in ipairs(BGL_TTM_VENDOR_NAMES) do
      if targetName == name then
        return true
      end
    end
  end

  return false
end

------------------------------------------------------------------------------------------------------------------------

function bglTTMEjectPassengersNow()
  print(BGL_TTM_COLOR_CODE.COLOR["Orange"]..L['PASSENGERS_EJECTED'])

  if ( bglTTMIsSeatAffected("Left") == true ) then
    EjectPassengerFromSeat(1)
  end

  if ( bglTTMIsSeatAffected("Right") == true ) then
    EjectPassengerFromSeat(2)
  end

  bglTTMSetEjectTabVisibility(false)
end

------------------------------------------------------------------------------------------------------------------------

function bglTTMSetEjectTabVisibility(enabled)
  if enabled then
    bglTTMMerchantFrameTab3:Show()
  else
    bglTTMMerchantFrameTab3:Hide()
  end
end

------------------------------------------------------------------------------------------------------------------------

function bglTTMIdentifyVehicleIndicatorFrame()
  local i = 0
  local frame = nil
  local frameList = GetFramesRegisteredForEvent("VEHICLE_PASSENGERS_CHANGED")

  for i=1,select("#",frameList) do
    if select(i,frameList):GetName() == "VehicleSeatIndicator" then
      return select(i,frameList)
    end
  end

  return nil
end

------------------------------------------------------------------------------------------------------------------------

function bglTTMInstallVehicleEjectButton()
  if not bglTTMVehicleEjectButton then
    local parentFrame = bglTTMIdentifyVehicleIndicatorFrame()
    bglTTMCreateVehicleEjectButton(parentFrame)
  end
end

------------------------------------------------------------------------------------------------------------------------
-- Event Handlers
------------------------------------------------------------------------------------------------------------------------

function bglTTMInterfaceOptionsPanel_OnLoad(frame)
  frame.name = BGL_TTM_ADDON_NAME

  frame.okay = function(self)
    bglTTMSaveData.autoEject = bglTTMInterfaceOptionsPanelAutoEjectOption:GetChecked()
    bglTTMSaveData.autoEjectAffectedSeats[1] = bglTTMInterfaceOptionsPanelAutoEjectAffectedSeat1Option:GetChecked()
    bglTTMSaveData.autoEjectAffectedSeats[2] = bglTTMInterfaceOptionsPanelAutoEjectAffectedSeat2Option:GetChecked()
    bglTTMSaveData.showVehicleEject = bglTTMInterfaceOptionsPanelShowVehicleOption:GetChecked()
    bglTTMSaveData.showVehicleEjectOnlyForTTMAndGEY = bglTTMInterfaceOptionsPanelLimitShowVehicleOption:GetChecked()
  end

  frame.refresh = function(self)
    bglTTMInterfaceOptionsPanelAutoEjectOption:SetChecked(bglTTMSaveData.autoEject == true)
    bglTTMInterfaceOptionsPanelAutoEjectAffectedSeat1Option:SetChecked(bglTTMIsSeatAffected("Left"))
    bglTTMInterfaceOptionsPanelAutoEjectAffectedSeat2Option:SetChecked(bglTTMIsSeatAffected("Right"))
    bglTTMInterfaceOptionsPanelShowVehicleOption:SetChecked(bglTTMSaveData.showVehicleEject == true)
    bglTTMInterfaceOptionsPanelLimitShowVehicleOption:SetChecked(bglTTMSaveData.showVehicleEjectOnlyForTTMAndGEY == true)
  end

  frame.default = function(self)
    bglTTMSaveData.autoEject = bglTTMData.default.autoEject
    bglTTMInterfaceOptionsPanelAutoEjectOption:SetChecked(bglTTMData.default.autoEject == true)

    bglTTMSaveData.autoEjectAffectedSeats = bglTTMData.default.autoEjectAffectedSeats
    bglTTMInterfaceOptionsPanelAutoEjectAffectedSeat1Option:SetChecked(true)
    bglTTMInterfaceOptionsPanelAutoEjectAffectedSeat2Option:SetChecked(true)

    bglTTMSaveData.showVehicleEject = bglTTMData.default.showVehicleEject
    bglTTMInterfaceOptionsPanelShowVehicleOption:SetChecked(bglTTMData.default.showVehicleEject == true)

    bglTTMSaveData.showVehicleEjectOnlyForTTMAndGEY = bglTTMData.default.showVehicleEjectOnlyForTTMAndGEY
    bglTTMInterfaceOptionsPanelLimitShowVehicleOption:SetChecked(bglTTMData.default.showVehicleEjectOnlyForTTMAndGEY == true)
  end

  InterfaceOptions_AddCategory(frame)
end

function bglTTM_OnEvent(self, event, ...)
  local playerName, _ = UnitName("player")

  if 
    event == "VEHICLE_PASSENGERS_CHANGED" 
    and ( bglTTMSaveData.autoEject == true )
    and ( bglTTMData.vendorsOnBoard > 0 )
  then
    bglTTMEjectPassengersNow()
    bglTTMData.vendorsOnBoard = bglTTMData.vendorsOnBoard - 1
  elseif event == "PLAYER_TARGET_CHANGED" then
    bglTTMSetEjectTabVisibility(bglTTMIsVendorPassenger())
  elseif event == "PLAYER_ENTERING_WORLD" then
    bglTTMInstallVehicleEjectButton()
  elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
    local casterGUID, spellName, spellID = ...

    if
      (
      (spellID == BGL_TTM_SPELL_ID1)
      or (spellID == BGL_TTM_SPELL_ID2)
      or (spellID == BGL_TTM_SPELL_ID3)
      or (spellID == BGL_TTM_SPELL_ID4)
      )
      and UnitIsUnit(casterGUID,"player") 
    then
      bglTTMSetEjectTabVisibility(false)
      bglTTMInstallVehicleEjectButton()
      bglTTMData.vendorsOnBoard = 2
      bglTTMData.mountIsTTMOrGEY = true
    else
      bglTTMData.vendorsOnBoard = 0
      bglTTMData.mountIsTTMOrGEY = false
    end

	-- if bglDebug is not null then print(BGL_TTM_COLOR_CODE.COLOR["Yellow"].."spellID: "..spellID) end

  end
end

------------------------------------------------------------------------------------------------------------------------
-- Addon Setup
------------------------------------------------------------------------------------------------------------------------

SLASH_bglttm1, SLASH_bglttm2 = "/bglttm","/ttm"
function SlashCmdList.bglttm(msg, editbox)
  local cmd, arg = strsplit(" ", strlower(msg))

  if cmd == L['COMMAND_HELP'] or cmd == "?" then
    print(BGL_TTM_COLOR_CODE.COLOR["Orange"]..L['CHAT_COMMANDS_HELP_DESCRIPTION'])
  elseif cmd == L['COMMAND_EJECT'] then
    bglTTMEjectPassengersNow()
  elseif cmd == L['COMMAND_AUTOEJECT'] or cmd == L['COMMAND_AUTO'] then
    -- default response is "on", so only check for "off"
    bglTTMSaveData.autoEject = (not arg) or not (arg == L['SUBCOMMAND_OFF'])
  elseif cmd == "r" then -- undocumented ReloadUI command
    ReloadUI()
  end
end
